#!/bin/perl
#
# Read output from the qsort testing program on stdin, generate summary
# table on stdout.

use Getopt::Std;
use strict;
use vars qw(%width %rtns %interesting $opt_t);

#$opt_t=1;

$interesting{'plain'} = 0;
$interesting{'alg1'} = 0;
$interesting{'alg2'} = 0;
$interesting{'alg3'} = 0;
$interesting{'system'} = 0;

# Compare hashes by 'Data Generator' first, then 'Array size',
# then 'Sort routine'
#
sub compare_hash {
   my $rval = ($a->{'Data Generator'} cmp $b->{'Data Generator'});
   if ($rval == 0) {
      $rval = ($a->{'Array size'} <=> $b->{'Array size'});
      if ($rval == 0) {
         $rval = ($a->{'Sort routine'} cmp $b->{'Sort routine'});
      }
   }
   return $rval;
}

# Collect each test as a hash of the <attribute> = <value> pairs it prints
# Return an array of the hashes.
#
sub collect_data {
   my $onetest;
   my @alltests;
   while (<>) {
      chomp;
      if (/^[ \t]*([^=]+) = (.+)$/) {
         if (! defined($onetest)) {
            $onetest = {};
         }
         $onetest->{$1} = $2;
         if (! defined($width{$1})) {
            $width{$1} = $2;
         } else {
            if (length($2) > length($width{$1})) {
               $width{$1} = $2;
            }
         }
      } else {
         if (defined($onetest)) {
            if (defined($onetest->{'Sort routine'})) {
               if (defined($interesting{$onetest->{'Sort routine'}})) {
                  $rtns{$onetest->{'Sort routine'}} = 1;
                  push(@alltests, $onetest);
               }
            }
         }
         $onetest = undef;
      }
   }
   if (defined($onetest)) {
      if (defined($onetest->{'Sort routine'})) {
         if (defined($interesting{$onetest->{'Sort routine'}})) {
            $rtns{$onetest->{'Sort routine'}} = 1;
            push(@alltests, $onetest);
         }
      }
   }
   @alltests = sort compare_hash (@alltests);
   return \@alltests;
}

# Sort data and print table
#
sub print_report {
   my $info = &collect_data;
   my $clock_ticks = $width{'Clock ticks'};
   if ($clock_ticks=~/^([0-9]+) /) {
      $clock_ticks = $1;
   }
   my $array_size = $width{'Array size'};
   my $compare_calls = $width{'Compare calls'};
   my $sort_routine = $width{'Sort routine'};
   $clock_ticks = length($clock_ticks);
   $array_size = length($array_size);
   $compare_calls = length($compare_calls);
   $sort_routine = length($sort_routine);
   if ($clock_ticks < length('ticks')) {
      $clock_ticks = length('ticks');
   }
   if ($compare_calls < length('cmps')) {
      $compare_calls = length('cmps');
   }
   if (! defined($opt_t)) {
      if ($sort_routine < $compare_calls) {
         $sort_routine = $compare_calls;
      }
      if ($compare_calls < $sort_routine) {
         $compare_calls = $sort_routine;
      }
   } else {
      if ($sort_routine < ($compare_calls + 1 + $clock_ticks)) {
         $sort_routine = ($compare_calls + 1 + $clock_ticks);
      }
      if (($compare_calls + 1 + $clock_ticks) < $sort_routine) {
         $compare_calls += ($sort_routine - ($compare_calls + 1 + $clock_ticks));
      }
   }
   if ($array_size < length('size')) {
      $array_size = length('size');
   }
   my $ctfmt = '%' . sprintf("%ds", $clock_ticks + 1);
   my $asfmt = '%' . sprintf("%ds", $array_size + 1);
   my $ccfmt = '%' . sprintf("%ds", $compare_calls + 1);
   my $srfmt = '%' . sprintf("%ds", $sort_routine + 1);
   my $dg;
   my $last_dg = '';
   my $last_as = '';
   my $h1 = sprintf($asfmt, '');
   my $h2 = sprintf($asfmt, 'size');
   my $h0 = "\n";
   my $r;
   foreach $r (sort(keys(%rtns))) {
      $h1 .= sprintf($srfmt, $r);
      $h2 .= sprintf($ccfmt, 'cmps');
      if (defined($opt_t)) {
         $h2 .= sprintf($ctfmt, 'ticks');
      }
   }
   foreach $r (@$info) {
      my $dg = $r->{'Data Generator'};
      my $as = $r->{'Array size'};
      my $cc = $r->{'Compare calls'};
      my $ct = $r->{'Clock ticks'};
      $interesting{$r->{'Sort routine'}} += $cc;
      if ($ct=~/^([0-9]+) /) {
         $ct = $1;
      }
      if ($dg ne $last_dg) {
         $last_dg = $dg;
         $last_as = '';
         print "$h0$dg test data:\n";
         $h0 = "\n\n";
         print "$h1\n";
         print "$h2\n";
      }
      if ($last_as ne $as) {
         $last_as = $as;
         print "\n";
         print sprintf($asfmt,$as);
      }
      print sprintf($ccfmt, $cc);
      if (defined($opt_t)) {
         print sprintf($ctfmt, $ct);
      }
   }
   print "\n\nOverall compare count stats:\n\n";
   foreach $r (sort { $interesting{$a} <=> $interesting{$b} }
                 (keys(%interesting))) {
      print sprintf("%23s = %d\n", $r, $interesting{$r});
   }
}

&print_report;
