#ifndef tqsort_h
#define tqsort_h

#include <stdlib.h>

/* sort routine prototype
*/
typedef void
sort_proto(void *, size_t, size_t, int (*)(const void *, const void *));

struct ftable_struct;

/* "base class" for sort tester objects
*/
typedef struct tester_struct {
   struct ftable_struct * ftable_ptr;
} tester;

/* Function table used to implement C++ like dispatching but still be able to
   build test program with just C
*/
typedef struct ftable_struct {
   /* public */
   void (*NewTest)(tester * tp, size_t num_elts);
   void (*TestSort)(tester * tp, sort_proto * sort_routine, const char * desc);
   void (*Destroy)(tester * tp);
   /* private */
   void (*Describe)(tester * tp);
} ftable;

#define NEW_TEST(tp, num_elts) \
   ((*(((tester *)tp)->ftable_ptr->NewTest))(((tester *)tp), (num_elts)))

#define TEST_SORT(tp, sort_routine, desc) \
   ((*(((tester *)tp)->ftable_ptr->TestSort)) \
      (((tester *)tp), (sort_routine), (desc)))

#define DESTROY(tp) \
   ((*(((tester *)tp)->ftable_ptr->Destroy))(((tester *)tp)))

#define DESCRIBE(tp) \
   ((*(((tester *)tp)->ftable_ptr->Describe))(((tester *)tp)))

/* "constructors" for different kinds of tester objects
*/
void InitTester(tester * tp);
void CleanTester(tester * tp);

tester * MakeRandom(size_t elt_size);
void InitRandom(tester * tp, size_t elt_size);
void CleanRandom(tester * tp);

tester * MakeAscending(size_t elt_size);
void InitAscending(tester * tp, size_t elt_size);
void CleanAscending(tester * tp);

tester * MakeDescending(size_t elt_size);
void InitDescending(tester * tp, size_t elt_size);
void CleanDescending(tester * tp);

tester * MakeRandup(size_t elt_size);
void InitRandup(tester * tp, size_t elt_size);
void CleanRandup(tester * tp);

tester * MakeScrambled(size_t elt_size);
void InitScrambled(tester * tp, size_t elt_size);
void CleanScrambled(tester * tp);

#endif /* tqsort_h */
