#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "trandom.h"

static random_test * the_tp = NULL;

static void * the_array = NULL;

static int
RawCompare(const void * ap, const void * bp)
{
   const char * a = ((const char *)ap);
   const char * b = ((const char *)bp);
   return memcmp((const void *)a, (const void *)b, the_tp->elt_size);
}

static int
StatsCompare(const void * ap, const void * bp)
{
   the_tp->compare_count += 1;
   return RawCompare(ap, bp);
}

static void
NewTest(tester * tp, size_t num_elts)
{
   random_test * rtp = (random_test *)tp;
   size_t i;

   rtp->num_elts = num_elts;
   if (rtp->orig_data != NULL) {
      free(rtp->orig_data);
   }
   rtp->orig_data = malloc(rtp->elt_size * rtp->num_elts);
   srand(0x1234beef);
   for (i = 0; i < rtp->num_elts; ++i) {
      char * eltp = ((char *)rtp->orig_data) + (i * rtp->elt_size);
      size_t elt_size = rtp->elt_size;
      unsigned long ul = 0;
      while (elt_size-- > 0) {
         if (ul == 0) {
            ul = (unsigned long)rand();
         }
         *eltp++ = (char)ul;
         ul >>= 8;
      }
   }
}

static void
TestSort(tester * tp, sort_proto * sort_routine, const char * desc)
{
   random_test * rtp = (random_test *)tp;
   int sorted;
   int i;
   clock_t start;
   clock_t stop;

   DESCRIBE(rtp);
   printf("   Sort routine = %s\n", desc);
   printf("     Array size = %lu\n", (unsigned long)rtp->num_elts);
   printf("   Element size = %lu\n", (unsigned long)rtp->elt_size);
   rtp->compare_count = 0;
   the_tp = rtp;
   the_array = malloc(rtp->num_elts * rtp->elt_size);
   memcpy(the_array, rtp->orig_data, rtp->num_elts * rtp->elt_size);
   fflush(stdout);
   (*sort_routine)(the_array, rtp->num_elts, rtp->elt_size, StatsCompare);
   printf("  Compare calls = %d\n", rtp->compare_count);
   sorted = 1;
   for (i = 1; i < (int)(rtp->num_elts); ++i) {
      char * im1 = ((char *)the_array) + (rtp->elt_size * (i - 1));
      char * im0 = ((char *)the_array) + (rtp->elt_size * (i - 0));
      if (RawCompare((const void *) im1, (const void *) im0) > 0) {
         sorted = 0;
      }
   }
   memcpy(the_array, rtp->orig_data, rtp->num_elts * rtp->elt_size);
   start = clock();
   if (start != (clock_t)-1) {
      double ms;
      unsigned long ticks;

      (*sort_routine)(the_array, rtp->num_elts, rtp->elt_size, RawCompare);
      stop = clock();
      ticks = ((unsigned long)stop) - ((unsigned long)start);
      ms = ((double)ticks) / ((double)CLOCKS_PER_SEC);
      ms *= 1000.0;
      printf("    Clock ticks = %lu (%.3f milliseconds)\n", ticks, ms);
   }
   if (! sorted) {
      printf("   * * * The array does not look sorted! * * *\n");
   }
   fflush(stdout);
   free(the_array);
}

static void
Destroy(tester * tp)
{
   CleanRandom(tp);
   free((void *)tp);
}

static void
Describe(tester * tp)
{
   printf("\n Data Generator = random\n");
}

tester *
MakeRandom(size_t elt_size)
{
   random_test * rtp = (random_test *)malloc(sizeof(random_test));
   InitRandom(&(rtp->base), elt_size);
   return &(rtp->base);
}

void
InitRandom(tester * tp, size_t elt_size)
{
   random_test * rtp = (random_test *)tp;
   InitTester(&(rtp->base));
   rtp->base.ftable_ptr->NewTest = NewTest;
   rtp->base.ftable_ptr->TestSort = TestSort;
   rtp->base.ftable_ptr->Destroy = Destroy;
   rtp->base.ftable_ptr->Describe = Describe;
   rtp->elt_size = elt_size;
   rtp->num_elts = 0;
   rtp->orig_data = NULL;
}

void
CleanRandom(tester * tp)
{
   random_test * rtp = (random_test *)tp;
   CleanTester(&(rtp->base));
   if (rtp->orig_data != NULL) {
      free(rtp->orig_data);
   }
}
